__author__ = 'gph82'
# Since the complicated stuff and the potential for undetectable errors is in bmutils, heavy testing should be done there. This is an API test for checking
# input parsing and decision making (if then) inside the visualize API
import unittest
import numpy as np
import molpx
from glob import glob
from molpx import visualize, _bmutils
import mdtraj as md
import matplotlib.pyplot as plt
import nglview
plt.switch_backend('Agg') # allow tests

from .test_bmutils import TestWithBPTIData

class TestTrajInputs(TestWithBPTIData):

    @classmethod
    def setUpClass(self):
        TestWithBPTIData.setUpClass()

        self.ala2_topology_file = molpx._molpxdir(join='notebooks/data/ala2.pdb')
        self.metad_trajectory_files = glob(molpx._molpxdir(join='notebooks/data/ala2.meta*.xtc'))
        self.metad_colvar_files = glob(molpx._molpxdir(join='notebooks/data/ala2.meta.CV.*txt'))

    def test_simplest_inputs_memory(self):
        visualize.traj(self.MD_trajectories, self.MD_topology, self.Ys)

    def test_simplest_inputs_memory_stride(self):
            visualize.traj(self.MD_trajectories, self.MD_topology, self.Ys, stride=2)

    def test_simplest_inputs_memory_small_max_frames(self):
            visualize.traj(self.MD_trajectories, self.MD_topology, self.Ys, max_frames=3)

    def test_simplest_inputs_disk(self):
        visualize.traj(self.MD_trajectory_files, self.MD_topology_file, self.projected_files)
        visualize.traj(self.MD_trajectory_files, self.MD_topology_file, [ifile.replace('.npy', '.dat') for ifile in self.projected_files])

    def test_simplest_inputs_memory_and_proj(self):
        visualize.traj(self.MD_trajectories, self.MD_topology, self.Ys, projection=self.tica)

    def test_simplest_inputs_memory_and_proj_just_matrix(self):
        visualize.traj(self.MD_trajectories, self.MD_topology, self.Ys,
                       projection=self.tica.feature_TIC_correlation)

    def test_simplest_inputs_memory_just_one_row(self):
        visualize.traj(self.MD_trajectories, self.MD_topology, self.Ys,
                       traj_selection=0,
                       proj_idxs=0)

    def test_simplest_inputs_memory_FES(self):
        visualize.traj(self.MD_trajectories, self.MD_topology, self.Ys,
                       plot_FES=True
                      )

    def test_fail_without_top(self):
        # This should fail
        try:
            visualize.traj(self.MD_trajectory_files, "unexisting_file", self.Ys)
        except (OSError, IOError):
            pass
        # This should pass
        visualize.traj(self.MD_trajectories, "unexisting_file", self.Ys)

    def test_listify_inputs(self):
        visualize.traj(self.MD_trajectories[0], self.MD_topology, self.Ys[0])

    def test_listify_params(self):
        visualize.traj(self.MD_trajectories, self.MD_topology, self.Ys, proj_idxs=1)

    def test_plotting_params(self):
        visualize.traj(self.MD_trajectories, self.MD_topology, self.Ys, sharey_traj=True)

    def test_fail_on_FES(self):
        try:
            fake_4D_Ys = [np.hstack((iY, iY)) for iY in self.Ys]
            visualize.traj(self.MD_trajectories, self.MD_topology,
                           fake_4D_Ys, sharey_traj=True,
                           plot_FES=True, proj_idxs=[0,1,2])
        except Exception as e:
            assert isinstance(e,NotImplementedError)

    def test_weights_on_biased_FES(self):
        weights = [np.exp(np.loadtxt(iw)[:,7]) for iw in self.metad_colvar_files]
        visualize.FES(self.metad_trajectory_files, self.ala2_topology_file, self.metad_colvar_files,
                      proj_idxs=[1,2], weights=weights)

class TestNGLWidgetWrapper(unittest.TestCase):

    def setUp(self):
        self.MD_file = molpx._molpxdir(join='notebooks/data/bpti-c-alpha_centered.pdb')
        self.MD_geom = md.load(self.MD_file)

    def test_widget_wrapper_w_None(self):
        iwd = molpx.visualize._nglwidget_wrapper(None)

    def test_widget_wrapper_w_file(self):
        iwd = molpx.visualize._nglwidget_wrapper(self.MD_file)

    def test_widget_wrapper_w_instantiated_wdg(self):
        iwd = molpx.visualize._nglwidget_wrapper(self.MD_file)
        molpx.visualize._nglwidget_wrapper(self.MD_geom, ngl_wdg=iwd)

def test_colors():
    _bmutils.matplotlib_colors_no_blue()

class TestSample(TestWithBPTIData):

    @classmethod
    def setUpClass(self):
        TestWithBPTIData.setUpClass()
        self.n_sample = 4
        self.pos = np.zeros((self.n_sample, 2))
        self.pos[:,0] = np.linspace(0,1,num=self.n_sample)
        self.pos[:,1] = np.random.rand(self.n_sample)
        self.geom = self.MD_trajectories[0][:self.n_sample]

    def test_sample_not_sticky_just_works(self):
        plt.figure()
        __ = molpx.visualize.sample(self.pos, self.geom, plt.gca())

    def test_sample_not_sticky_just_works_with_path(self):
        plt.figure()
        __ = molpx.visualize.sample(self.pos, self.geom, plt.gca(), plot_path=True)

    def test_sample_not_sticky_just_works_with_projection(self):
        plt.figure()
        __ = molpx.visualize.sample(self.pos, self.geom, plt.gca(), projection=self.tica)

    def test_sample_not_sticky_smooth(self):
        plt.figure()
        __ = molpx.visualize.sample(self.pos, self.geom, plt.gca(),
                                    n_smooth=1)

    def test_sample_sticky_just_works_one_geom(self):
        plt.figure()
        __ = molpx.visualize.sample(self.pos, self.geom, plt.gca(), sticky=True)

    def test_sample_sticky_just_works_list_geom(self):
        plt.figure()
        __ = molpx.visualize.sample(self.pos, [self.geom, self.geom[::-1]], plt.gca(), sticky=True)

    def test_sample_sticky_just_works_list_geom_dont_orient(self):
        plt.figure()
        __ = molpx.visualize.sample(self.pos, [self.geom, self.geom[::-1]], plt.gca(),
                                    superpose=False,
                                    sticky=True)

    def test_sample_sticky_just_works_list_geom_colors(self):
        plt.figure()
        __ = molpx.visualize.sample(self.pos, [self.geom, self.geom[::-1]], plt.gca(),
                                    color_list=['r', 'b', 'g', 'magenta'],
                                    sticky=True)

        __ = molpx.visualize.sample(self.pos, [self.geom, self.geom[::-1]], plt.gca(),
                                    color_list='rand',
                                    sticky=True)

        try:
            __ = molpx.visualize.sample(self.pos, [self.geom, self.geom[::-1]], plt.gca(),
                                        color_list=1,
                                        sticky=True)
        except TypeError:
            pass

    def test_sample_sticky_just_works_list_geom_small_molecule(self):
        __ = molpx.visualize.sample(self.pos, [self.geom.atom_slice(np.arange(5)),
                                               self.geom[::-1].atom_slice(np.arange(5))],
                                    plt.gca(),
                                    color_list=['r', 'b', 'g', 'magenta'],
                                    sticky=True)

class TestFES(TestWithBPTIData):

    @classmethod
    def setUpClass(self):
        TestWithBPTIData.setUpClass()

    def test_just_works_min_input_disk(self):
        molpx.visualize.FES(self.MD_trajectory_files,
                            self.MD_topology_file,
                            self.projected_files)

    def test_just_works_min_input_memory(self):
        molpx.visualize.FES(self.MD_trajectories,
                            self.MD_topology,
                            self.Ys)

    def test_overlays(self):
        molpx.visualize.FES(self.MD_trajectories,
                            self.MD_topology,
                            self.Ys,
                            n_overlays=5)

    def test_1D(self):
        molpx.visualize.FES(self.MD_trajectories,
                            self.MD_topology,
                            self.Ys,
                            proj_idxs=[0])

class TestCorrelations(TestWithBPTIData):

    @classmethod
    def setUpClass(self):
        TestWithBPTIData.setUpClass()

    def test_correlations_input_tica(self):
        visualize.correlations(self.tica)

    def test_correlations_input_pca(self):
        visualize.correlations(self.pca)

    def test_correlations_input_feat(self):
        visualize.correlations(self.feat)

    def test_correlations_input_tica_and_geoms(self):
        visualize.correlations(self.tica, geoms=self.MD_trajectories[0])

    def test_correlations_input_warn(self):
        visualize.correlations(self.tica.feature_TIC_correlation, geoms=self.MD_trajectories[0])

    def test_correlations_inputs_verbose(self):
        visualize.correlations(self.tica, verbose=True)

    def test_correlations_inputs_color_list_parsing(self):
        visualize.correlations(self.tica, proj_color_list=['green'])

    def test_correlations_inputs_FAIL_color_list_parsing(self):
        try:
            visualize.correlations(self.tica, proj_color_list='green')
        except TypeError:
            pass

class TestFeature(TestWithBPTIData):

    @classmethod
    def setUpClass(self):
        TestWithBPTIData.setUpClass()

    def test_feature(self):
        plt.figure()
        iwd = nglview.show_mdtraj(self.MD_trajectories[0])
        visualize.feature(self.feat.active_features[0], iwd)

    def test_feature_color_list(self):
        plt.figure()
        iwd = nglview.show_mdtraj(self.MD_trajectories[0])
        visualize.feature(self.feat.active_features[0], iwd,
                          idxs=[0,1],
                          color_list=['blue'])
        try:
            visualize.feature(self.feat.active_features[0], iwd,
                              idxs=[0,1],
                              color_list='blue')
        except TypeError:
            pass


class TestBoxMe(unittest.TestCase):

    def test_just_runs_and_exits_gracefully(self):
        from matplotlib.widgets import AxesWidget
        from ipywidgets.widgets import HBox
        ifig = plt.figure()
        iax = plt.gca()
        nglwdg = nglview.demo()
        mplw = AxesWidget(iax)

        box_out = visualize._box_me((ifig, iax, nglwdg,
                                     mplw
                                     ))

        assert isinstance(box_out, HBox)


        assert None is visualize._box_me((ifig, iax, nglwdg,
                                     mplw, 1
                                     ))


if __name__ == '__main__':
    unittest.main()



__author__ = 'gph82'
# Since the complicated stuff and the potential for undetectable errors is in bmutils, heavy testing should be done there. This is an API test for checking
# input parsing and decision making (if then) inside the visualize API
import unittest
import numpy as np
import molpx
from glob import glob
from matplotlib.backend_bases import MouseEvent
import mdtraj as md
import matplotlib.pyplot as plt
plt.switch_backend('Agg') # allow tests


import nglview
from scipy.spatial import cKDTree as _cKDTree



class TestLinkAxWPos2NGLWidget(unittest.TestCase):

    @classmethod
    def setUpClass(self):
        self.MD_trajectory_files = glob(molpx._molpxdir(join='notebooks/data/c-alpha_centered.stride.1000*xtc'))
        self.MD_topology_file = molpx._molpxdir(join='notebooks/data/bpti-c-alpha_centered.pdb')
        self.MD_topology = md.load(self.MD_topology_file).top
        self.MD_trajectories = [md.load(ff, top=self.MD_topology_file) for ff in self.MD_trajectory_files]

        self.n_sample = 20
        self.pos = np.zeros((self.n_sample,2))
        self.pos[:,0] = np.linspace(0,1,self.n_sample)
        self.pos[:,1] = np.random.randn(self.n_sample)
        self.geom = self.MD_trajectories[0][:20]
        self.ngl_wdg = nglview.show_mdtraj(self.geom)

    def test_just_works(self):
        plt.figure()
        __ = molpx._linkutils.link_ax_w_pos_2_nglwidget(plt.gca(), self.pos, self.ngl_wdg)

    def test_just_works_bandwidth(self):
        plt.figure()
        __ = molpx._linkutils.link_ax_w_pos_2_nglwidget(plt.gca(), self.pos, self.ngl_wdg,
                                                        band_width=[0.1, .1])
    def test_just_works_exclude_coord(self):
        plt.figure()
        __ = molpx._linkutils.link_ax_w_pos_2_nglwidget(plt.gca(), self.pos, self.ngl_wdg,
                                                        exclude_coord=1)

    def test_force_exceptions(self):
        plt.figure()
        try:
            __ = molpx._linkutils.link_ax_w_pos_2_nglwidget(plt.gca(), self.pos, self.ngl_wdg,
                                                            dot_color=2)
        except TypeError:
            pass

        plt.figure()
        try:
            pos_rnd = np.random.randn(self.n_sample, 2)
            __ = molpx._linkutils.link_ax_w_pos_2_nglwidget(plt.gca(), pos_rnd, self.ngl_wdg,
                                                            band_width=[.1, .1])
        except ValueError:
            pass

    def test_just_works_radius(self):
        plt.figure()
        __ = molpx._linkutils.link_ax_w_pos_2_nglwidget(plt.gca(), self.pos, self.ngl_wdg,
                                                        band_width=[.1, .1],
                                                        radius=True)
class TestGeometryInNGLWidget(unittest.TestCase):

    # TODO abstact this to a test class
    @classmethod
    def setUpClass(self):
        self.MD_trajectory_files = glob(molpx._molpxdir(join='notebooks/data/c-alpha_centered.stride.1000*xtc'))
        self.MD_topology_file = molpx._molpxdir(join='notebooks/data/bpti-c-alpha_centered.pdb')
        self.MD_topology = md.load(self.MD_topology_file).top
        self.MD_trajectories = [md.load(ff, top=self.MD_topology_file) for ff in self.MD_trajectory_files]

        self.n_sample = 20
        self.pos = np.zeros((self.n_sample,2))
        self.pos[:,0] = np.linspace(0,1,self.n_sample)
        self.pos[:,1] = np.random.randn(self.n_sample)
        self.geom = self.MD_trajectories[0][:20]
        self.ngl_wdg = nglview.show_mdtraj(self.geom)

    def test_just_works(self):
        __ = molpx._linkutils.GeometryInNGLWidget(self.geom, self.ngl_wdg)
        __ = molpx._linkutils.GeometryInNGLWidget(self.geom, self.ngl_wdg, n_small=500)


    def test_quickhands(self):
        giw = molpx._linkutils.GeometryInNGLWidget(self.geom, self.ngl_wdg)
        assert giw.is_empty()
        assert giw.have_repr == []
        assert ~giw.all_reps_are_on()
        assert giw.all_reps_are_off()
        assert ~giw.any_rep_is_on()
        assert ~giw.is_visible()


    def test_show_just_runs_and_changes_quickhands(self):
        giw = molpx._linkutils.GeometryInNGLWidget(self.geom, self.ngl_wdg)
        giw.show()
        assert ~giw.is_empty()
        assert len(giw.have_repr) == 1
        assert giw.all_reps_are_on()
        assert ~giw.all_reps_are_off()
        assert giw.any_rep_is_on()
        assert giw.is_visible()

    def test_show_and_hide_just_runs_and_changes_quickhands(self):
        # We initialize with two frames, it's easy to test the ends
        giw = molpx._linkutils.GeometryInNGLWidget(self.geom[:2], self.ngl_wdg)
        giw.show()
        giw.hide()
        assert ~giw.is_empty()
        assert len(giw.have_repr) == 1 #should still be one
        assert ~giw.all_reps_are_on() # but it´s off
        assert giw.all_reps_are_off()
        assert ~giw.any_rep_is_on()
        assert ~giw.is_visible()
        assert giw.any_rep_is_off()
        # Now we show again
        giw.show() # Turn on the one we had already, does not change the have_repr
        assert len(giw.have_repr)==1
        giw.show()
        assert len(giw.have_repr)==2
        # Turn on a new one, which isn't there nothing should happen
        [giw.show() for ii in range(10)]
        # Now we hide many times, should arrive at the end without anything happening
        [giw.hide() for ii in range(10)]

class TestUpdate2DLines(unittest.TestCase):

    def test_all(self):
        plt.plot(0,0)
        line = plt.gca().lines[0]
        for ii, attr in enumerate(['lineh', 'linev', 'dot']):
            setattr(line,"whatisthis",attr)
            molpx._linkutils.update2Dlines(line, ii, ii)

    def test_force_exceptions(self):
        plt.plot(0, 0)
        line = plt.gca().lines[0]
        try:
            molpx._linkutils.update2Dlines(line,0,0)
        except AttributeError:
            pass

        setattr(line, "whatisthis", "non_existing_line")
        try:
            molpx._linkutils.update2Dlines(line,0,0)
        except TypeError:
            pass

class TestClickOnAxisListener(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.MD_trajectory_files = glob(molpx._molpxdir(join='notebooks/data/c-alpha_centered.stride.1000*xtc'))
        self.MD_topology_file = molpx._molpxdir(join='notebooks/data/bpti-c-alpha_centered.pdb')
        self.MD_topology = md.load(self.MD_topology_file).top
        self.MD_trajectories = [md.load(ff, top=self.MD_topology_file) for ff in self.MD_trajectory_files]
        self.MD_trajectory = self.MD_trajectories[0]

        self.pos = np.random.rand(self.MD_trajectory.n_frames, 2)
        self.kdtree = _cKDTree(self.pos)

    def just_runs(self, ngl_wdg, button=None):
        # Create the linked objects
        plt.plot(self.pos[:,0], self.pos[:,1])
        iax = plt.gca()

        # Prepare a mouse event in the middle of the plot
        x, y = np.array(iax.get_window_extent()).mean(0)

        # Prepare event
        lineh = iax.axhline(iax.get_ybound()[0])
        setattr(lineh, 'whatisthis', 'lineh')
        dot = iax.plot(self.pos[0, 0], self.pos[0, 1])[0]
        setattr(dot, 'whatisthis', 'dot')

        # Instantiate the ClickOnAxisListener and call it with the event
        return molpx._linkutils.ClickOnAxisListener(ngl_wdg, True,
                                             [lineh],
                                             iax, self.pos,
                                             [dot]
                                             )(MouseEvent(" ", plt.gcf().canvas, x,y,
                                                          button=button, key=None, step=0,
                                                          dblclick=False,
                                                          guiEvent=None))

    def test_just_runs(self):
        self.just_runs(nglview.show_mdtraj(self.MD_trajectory))

    def test_just_runs_sticky(self):
        # Create the linked objects (for sticky case, better use _sample
        ngl_wdg, __ = molpx.visualize.sample(self.pos, self.MD_trajectory, plt.gca(), sticky=True)
        self.just_runs(ngl_wdg, button=1)
        [self.just_runs(ngl_wdg, button=2) for ii in range(5)]

    def test_just_runs_recomputes_kdtree(self):
        plt.plot(self.pos[:, 0], self.pos[:, 1])
        iax = plt.gca()

        # Prepare a mouse event in the middle of the plot
        x, y = np.array(iax.get_window_extent()).mean(0)

        # Prepare event
        lineh = iax.axhline(iax.get_ybound()[0])
        setattr(lineh, 'whatisthis', 'lineh')
        dot = iax.plot(self.pos[0, 0], self.pos[0, 1])[0]
        setattr(dot, 'whatisthis', 'dot')
        CLAL = molpx._linkutils.ClickOnAxisListener(nglview.show_mdtraj(self.MD_trajectory), True,
                                             [lineh],
                                             iax, self.pos,
                                             [dot]
                                             )
        # Resize the figure
        CLAL.ax.figure.set_size_inches(1,1)
        # Send an event
        CLAL(MouseEvent(" ",  CLAL.ax.figure.canvas, x, y,
                    button=1, key=None, step=0,
                    dblclick=False,
                    guiEvent=None))

class TestChangeInNGLWidgetListener(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.MD_trajectory_files = glob(molpx._molpxdir(join='notebooks/data/c-alpha_centered.stride.1000*xtc'))
        self.MD_topology_file = molpx._molpxdir(join='notebooks/data/bpti-c-alpha_centered.pdb')
        self.MD_topology = md.load(self.MD_topology_file).top
        self.MD_trajectories = [md.load(ff, top=self.MD_topology_file) for ff in self.MD_trajectory_files]
        self.MD_trajectory = self.MD_trajectories[0]

        self.pos = np.random.rand(self.MD_trajectory.n_frames, 2)
        plt.plot(self.pos[:,0], self.pos[:,1])
        iax = plt.gca()
        # Prepare event
        self.lineh = iax.axhline(iax.get_ybound()[0])
        setattr(self.lineh, 'whatisthis', 'lineh')
        self.dot = iax.plot(self.pos[0, 0], self.pos[0, 1])[0]
        setattr(self.dot, 'whatisthis', 'dot')
        self.ngl_wdg = nglview.show_mdtraj(self.MD_trajectory)
    def test_just_runs(self):
        molpx._linkutils.ChangeInNGLWidgetListener(self.ngl_wdg, [self.lineh, self.dot], self.pos)({"new":0})

    def test_just_runs_past_last_frame(self):
        molpx._linkutils.ChangeInNGLWidgetListener(self.ngl_wdg, [self.lineh, self.dot], self.pos)({"new":self.pos.shape[0]+1,
                                                                                                    "old":1})

if __name__ == '__main__':
    unittest.main()



r"""
=====================================
molPX - Molecular Projection Explorer
=====================================
"""
from __future__ import print_function as _

__author__ = 'gph82'

from . import generate
from . import visualize
from . import _bmutils

from ._nbtools import example_notebooks, _molpxdir

def _report_status():
    r"""
    returns a boolen whether molpx is allowed to send user metadata
    """
    import os,sys
    conf_module = os.path.expanduser('~/.molpx')
    sys.path.append(conf_module)
    res = True
    try:
        import conf_molpx
        try:
            res =  conf_molpx.report_status
            if not isinstance(res, bool):
                pass
                #TODO write warning about malformed conf_molpx.py file
        except AttributeError:
            pass
            #TODO write warning about corrupted conf_molpx.py file
    except ImportError:
         #TODO write warning about corrupted conf_molpx.py file
        pass
    except NameError:
        #TODO write warning about corrupted conf_molpx.py file
        pass

    sys.path = sys.path[:-1]

    return res

def _version_check(current, testing=False):
    """ checks latest version online from the server and logs some user metadata.
    Can be disabled by setting config.check_version = False.

    """

    import platform
    import os
    from six.moves.urllib.request import urlopen, Request
    from contextlib import closing
    import threading
    import uuid

    import sys
    if 'pytest' in sys.modules or os.getenv('CI', False):
        testing = True

    def _impl():
        try:
            r = Request('http://emma-project.org/versions.json',
                        headers={'User-Agent': 'molpx-{molpx_version}-Py-{python_version}-{platform}-{addr}'
                        .format(molpx_version=current, python_version=platform.python_version(),
                                platform=platform.platform(terse=True), addr=uuid.getnode())} if not testing else {})
            with closing(urlopen(r, timeout=30)):
                pass
            """
            versions = json.loads(payload)
            latest_json = tuple(filter(lambda x: x['latest'], versions))[0]['version']
            latest = parse(latest_json)
            if parse(current) < latest:
                import warnings
                warnings.warn("You are not using the latest release of PyEMMA."
                              " Latest is {latest}, you have {current}."
                              .format(latest=latest, current=current), category=UserWarning)
            """
            # TODO add warnings about versions
        except Exception:
            pass
            """
            import logging
            logging.getLogger('pyemma').debug("error during version check", exc_info=True)
            """
            # TODO add loggers

    return threading.Thread(target=_impl if _report_status() and not testing else lambda: '')

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions

# start check in background
_version_check(__version__).start()

#
import subprocess as _subprocess
import warnings as _warnings


# The name by which an extension appears as enabled or not is different
# from the one used to install it (!!),so we need a mapping
_ext_mapping = {"nglview-js-widgets": "nglview",
                "jupyter-matplotlib": "ipympl",
                "jupyter-js-widgets": "widgetsnbextension"
                }

def _get_extension_status(ext_list=_ext_mapping.keys()):
    r"""
    Guess the status of the extensions in ext_list. the correct way of doing this
    would be using notebook.nbextensions, but you need paths to the extensions
    and I dont want to go there
    :return: dictionary with extensions in ext_list as keys and True or False as values
    """
    enabled_exts = {key:None for key in ext_list}

    lines = _subprocess.check_output(("jupyter-nbextension", "list"), stderr=_subprocess.DEVNULL)
    for ext in enabled_exts.keys():
        for iline in lines.decode().splitlines():
            if ext in iline and "disabled" in iline.lower():
                enabled_exts[ext] = False
            elif ext in iline and "enabled" in iline.lower():
                if enabled_exts[ext] is None:
                    enabled_exts[ext] = True
                enabled_exts[ext] = enabled_exts[ext] and True

    for key in enabled_exts.keys():
        if enabled_exts[key] is None:
            enabled_exts[key] = False

    return enabled_exts

def _enable_extensions(this_ext_path):
    r""" Try to install/enable an extension.
    Prompt the user to do so if an exception is thrown
    """

    try:
        _subprocess.check_call([
            'jupyter', 'nbextension', 'install', '--py',
            '--sys-prefix', this_ext_path
        ], stderr=_subprocess.DEVNULL)

        _subprocess.check_call([
            'jupyter', 'nbextension', 'enable', '--py',
            '--sys-prefix', this_ext_path
        ], stderr=_subprocess.DEVNULL)
    except:
        _warnings.warn("\nWe could not automatically enable the extention %s.\n"
                       "From the command line type:\n jupyter-nbextension enable %s --py --sys-prefix" % (this_ext_path, this_ext_path))
        return False
    return True

# Try to help the user getting molpx working out of the box and raise an Exception if molpx wont work
if not all(_get_extension_status().values()):
    for ext, enabled in _get_extension_status().items():
        if not enabled:
            if not _enable_extensions(_ext_mapping[ext]):
                raise ModuleNotFoundError("Could not initialize molpx")

